@@ -61,8 +61,7 @@ bool Parameter::initType(
     auto builtinType = stringToBuiltinType(typeName);
     if (builtinType == QV4::CompiledData::CommonType::Invalid) {
         if (typeName.isEmpty()) {
-            paramType->set(
-                    listFlag | QV4::CompiledData::ParameterType::Common, quint32(builtinType));
+            paramType->set(listFlag, 0);
             return false;
         }
         Q_ASSERT(quint32(typeNameIndex) < (1u << 31));
@@ -443,38 +442,6 @@ bool IRBuilder::generateFromQml(const QString &code, const QString &url, Documen
     return errors.isEmpty();
 }
 
-bool IRBuilder::isSignalPropertyName(const QString &name)
-{
-    if (name.size() < 3) return false;
-    if (!name.startsWith(QLatin1String("on"))) return false;
-    int ns = name.size();
-    for (int i = 2; i < ns; ++i) {
-        const QChar curr = name.at(i);
-        if (curr.unicode() == '_') continue;
-        if (curr.isUpper()) return true;
-        return false;
-    }
-    return false; // consists solely of underscores - invalid.
-}
-
-QString IRBuilder::signalNameFromSignalPropertyName(const QString &signalPropertyName)
-{
-    Q_ASSERT(signalPropertyName.startsWith(QLatin1String("on")));
-    QString signalNameCandidate = signalPropertyName;
-    signalNameCandidate.remove(0, 2);
-
-    // Note that the property name could start with any alpha or '_' or '$' character,
-    // so we need to do the lower-casing of the first alpha character.
-    for (int firstAlphaIndex = 0; firstAlphaIndex < signalNameCandidate.size(); ++firstAlphaIndex) {
-        if (signalNameCandidate.at(firstAlphaIndex).isUpper()) {
-            signalNameCandidate[firstAlphaIndex] = signalNameCandidate.at(firstAlphaIndex).toLower();
-            return signalNameCandidate;
-        }
-    }
-
-    Q_UNREACHABLE_RETURN(QString());
-}
-
 bool IRBuilder::visit(QQmlJS::AST::UiArrayMemberList *ast)
 {
     return QQmlJS::AST::Visitor::visit(ast);
@@ -1718,10 +1685,9 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
             case Pragma::FunctionSignatureBehavior:
                 switch (p->functionSignatureBehavior) {
                 case Pragma::Enforced:
-                    createdUnit->flags |= Unit::FunctionSignaturesEnforced;
                     break;
                 case Pragma::Ignored:
-                    //this is the default;
+                    createdUnit->flags |= Unit::FunctionSignaturesIgnored;
                     break;
                 }
                 break;
