@@ -3,6 +3,7 @@
 
 #include "qquickabstractbutton_p.h"
 #include "qquickabstractbutton_p_p.h"
+#include "qquickactiongroup_p.h"
 #include "qquickbuttongroup_p.h"
 #include "qquickaction_p.h"
 #include "qquickaction_p_p.h"
@@ -62,6 +63,8 @@ QT_BEGIN_NAMESPACE
     \qmlsignal QtQuick.Controls::AbstractButton::clicked()
 
     This signal is emitted when the button is interactively clicked by the user via touch, mouse, or keyboard.
+
+    \sa {Call a C++ function from QML when a Button is clicked}
 */
 
 /*!
@@ -416,7 +419,7 @@ QQuickAbstractButton *QQuickAbstractButtonPrivate::findCheckedButton() const
 {
     Q_Q(const QQuickAbstractButton);
     if (group)
-        return qobject_cast<QQuickAbstractButton *>(group->checkedButton());
+        return group->checkedButton();
 
     const QList<QQuickAbstractButton *> buttons = findExclusiveButtons();
     // TODO: A singular QRadioButton can be unchecked, which seems logical,
@@ -497,8 +500,14 @@ QQuickAbstractButton::~QQuickAbstractButton()
 {
     Q_D(QQuickAbstractButton);
     d->removeImplicitSizeListener(d->indicator);
-    if (d->group)
-        d->group->removeButton(this);
+    if (d->group) {
+        auto *attached = qobject_cast<QQuickButtonGroupAttached *>(
+            qmlAttachedPropertiesObject<QQuickButtonGroup>(this, false));
+        if (attached)
+            attached->setGroup(nullptr);
+        else
+            d->group->removeButton(this);
+    }
 #if QT_CONFIG(shortcut)
     d->ungrabShortcut();
 #endif
@@ -1117,6 +1126,11 @@ void QQuickAbstractButton::keyReleaseEvent(QKeyEvent *event)
 
 void QQuickAbstractButton::mousePressEvent(QMouseEvent *event)
 {
+    if (!(event->buttons() & Qt::LeftButton)) {
+        event->ignore();
+        return;
+    }
+
     Q_D(QQuickAbstractButton);
     d->pressButtons = event->buttons();
     QQuickControl::mousePressEvent(event);
@@ -1124,9 +1138,10 @@ void QQuickAbstractButton::mousePressEvent(QMouseEvent *event)
 
 void QQuickAbstractButton::mouseDoubleClickEvent(QMouseEvent *event)
 {
+    Q_UNUSED(event);
     Q_D(QQuickAbstractButton);
     if (d->isDoubleClickConnected()) {
-        QQuickControl::mouseDoubleClickEvent(event);
+        // don't call QQuickItem::mouseDoubleClickEvent(): it would ignore()
         emit doubleClicked();
         d->wasDoubleClick = true;
     }
@@ -1191,8 +1206,21 @@ void QQuickAbstractButton::buttonChange(ButtonChange change)
 void QQuickAbstractButton::nextCheckState()
 {
     Q_D(QQuickAbstractButton);
-    if (d->checkable && (!d->checked || d->findCheckedButton() != this))
-        d->toggle(!d->checked);
+    if (!d->checkable)
+        return;
+
+    if (d->checked) {
+        if (d->findCheckedButton() == this)
+            return;
+        if (d->action) {
+            // For non-exclusive groups checkedAction is null
+            if (const auto group = QQuickActionPrivate::get(d->action)->group)
+                if (group->checkedAction() == d->action)
+                    return;
+        }
+    }
+
+    d->toggle(!d->checked);
 }
 
 #if QT_CONFIG(accessibility)
