@@ -356,6 +356,12 @@ void tst_qmltyperegistrar::addRemoveVersion()
     QCOMPARE(o->property("thing").toInt(), thingAccessible ? 24 : 0);
 }
 
+void tst_qmltyperegistrar::addInMinorVersion()
+{
+    QVERIFY(qmltypesData.contains("exports: [\"QmlTypeRegistrarTest/MinorVersioned 1.5\"]"));
+    QVERIFY(qmltypesData.contains("exports: [\"QmlTypeRegistrarTest/MinorVersioned 1.2\"]"));
+}
+
 #ifdef QT_QUICK_LIB
 void tst_qmltyperegistrar::foreignRevisionedProperty()
 {
@@ -398,8 +404,8 @@ void tst_qmltyperegistrar::duplicateExportWarnings()
     MetaTypesJsonProcessor processor(true);
     QVERIFY(processor.processTypes({ ":/duplicatedExports.json" }));
     processor.postProcessTypes();
-    QVector<QJsonObject> types = processor.types();
-    QVector<QJsonObject> typesforeign = processor.foreignTypes();
+    QVector<QCborMap> types = processor.types();
+    QVector<QCborMap> typesforeign = processor.foreignTypes();
     r.setTypes(types, typesforeign);
 
     auto expectWarning = [](QString message) {
@@ -536,13 +542,15 @@ void tst_qmltyperegistrar::uncreatable()
 #if QT_DEPRECATED_SINCE(6, 4)
     QTest::ignoreMessage(
                 QtWarningMsg,
-                "BadUncreatable is neither a QObject, nor default- and copy-constructible, "
-                "nor uncreatable. You should not use it as a QML type.");
+                "BadUncreatable is neither a default constructible QObject, nor a default- "
+                "and copy-constructible Q_GADGET, nor marked as uncreatable.\n"
+                "You should not use it as a QML type.");
     qmlRegisterTypesAndRevisions<BadUncreatable>("A", 1);
     QTest::ignoreMessage(
                 QtWarningMsg,
-                "BadUncreatableExtended is neither a QObject, nor default- and copy-constructible, "
-                "nor uncreatable. You should not use it as a QML type.");
+                "BadUncreatableExtended is neither a default constructible QObject, nor a default- "
+                "and copy-constructible Q_GADGET, nor marked as uncreatable.\n"
+                "You should not use it as a QML type.");
     qmlRegisterTypesAndRevisions<BadUncreatableExtended>("A", 1);
 #endif
 
@@ -573,6 +581,38 @@ void tst_qmltyperegistrar::baseVersionInQmltypes()
     QVERIFY(qmltypesData.contains("exports: [\"QmlTypeRegistrarTest/WithMethod 1.0\"]"));
 }
 
+void tst_qmltyperegistrar::constructibleValueType()
+{
+    QVERIFY(qmltypesData.contains(
+    R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "Constructible"
+        accessSemantics: "value"
+        exports: ["QmlTypeRegistrarTest/constructible 1.0"]
+        exportMetaObjectRevisions: [256]
+        Method {
+            name: "Constructible"
+            isConstructor: true
+            Parameter { name: "i"; type: "int" }
+        }
+        Method { name: "Constructible"; isCloned: true; isConstructor: true }
+    })"));
+}
+
+void tst_qmltyperegistrar::structuredValueType()
+{
+    QVERIFY(qmltypesData.contains(
+    R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "Structured"
+        accessSemantics: "value"
+        exports: ["QmlTypeRegistrarTest/structured 1.0"]
+        isStructured: true
+        exportMetaObjectRevisions: [256]
+        Property { name: "i"; type: "int"; index: 0; isFinal: true }
+    })"));
+}
+
 void tst_qmltyperegistrar::anonymousAndUncreatable()
 {
     QVERIFY(qmltypesData.contains(
@@ -588,7 +628,128 @@ void tst_qmltyperegistrar::omitInvisible()
 {
     // If it cannot resolve the type a QML_FOREIGN refers to, it should not generate anything.
     QVERIFY(qmltypesData.contains(
-    R"(Component { file: "tst_qmltyperegistrar.h"; name: "Invisible"; accessSemantics: "none" })"));
+                R"(Component { file: "tst_qmltyperegistrar.h"; name: "Invisible"; accessSemantics: "none" })"));
+}
+
+void tst_qmltyperegistrar::typedEnum()
+{
+    QVERIFY(qmltypesData.contains(
+    R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "TypedEnum"
+        accessSemantics: "reference"
+        prototype: "QObject"
+        exports: ["QmlTypeRegistrarTest/TypedEnum 1.0"]
+        exportMetaObjectRevisions: [256]
+        Enum {
+            name: "S"
+            type: "short"
+            values: ["A", "B", "C"]
+        }
+        Enum {
+            name: "T"
+            type: "ushort"
+            values: ["D", "E", "F"]
+        }
+        Enum {
+            name: "U"
+            type: "qint8"
+            values: ["G", "H", "I"]
+        }
+        Enum {
+            name: "V"
+            type: "quint8"
+            values: ["J", "K", "L"]
+        }
+    })"));
+}
+
+void tst_qmltyperegistrar::listSignal()
+{
+    QVERIFY(qmltypesData.contains(
+    R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "ListSignal"
+        accessSemantics: "reference"
+        prototype: "QObject"
+        Signal {
+            name: "objectListHappened"
+            Parameter { type: "QObjectList" }
+        }
+    })"));
+}
+
+void tst_qmltyperegistrar::withNamespace()
+{
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "Bar"
+        accessSemantics: "reference"
+        prototype: "QObject"
+        Property {
+            name: "outerBarProp"
+            type: "int"
+            read: "bar"
+            index: 0
+            isReadonly: true
+            isConstant: true
+        }
+    })"));
+
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "Testing::Bar"
+        accessSemantics: "reference"
+        prototype: "Testing::Foo"
+        exports: ["QmlTypeRegistrarTest/Bar 1.0"]
+        exportMetaObjectRevisions: [256]
+        Property { name: "barProp"; type: "int"; read: "bar"; index: 0; isReadonly: true; isConstant: true }
+    })"));
+
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "Testing::Foo"
+        accessSemantics: "reference"
+        prototype: "QObject"
+        Property { name: "fooProp"; type: "int"; read: "foo"; index: 0; isReadonly: true; isConstant: true }
+    })"));
+
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "Testing::Inner::Baz"
+        accessSemantics: "reference"
+        prototype: "Testing::Bar"
+        extension: "Bar"
+        exports: ["QmlTypeRegistrarTest/Baz 1.0"]
+        exportMetaObjectRevisions: [256]
+        attachedType: "Testing::Foo"
+    })"));
+}
+
+void tst_qmltyperegistrar::sequenceRegistration()
+{
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "std::vector<QByteArray>"
+        accessSemantics: "sequence"
+        valueType: "QByteArray"
+    })"));
+}
+
+void tst_qmltyperegistrar::valueTypeSelfReference()
+{
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "QPersistentModelIndex"
+        accessSemantics: "value"
+        extension: "QPersistentModelIndexValueType"
+    })"));
+    QVERIFY(qmltypesData.contains(R"(Component {
+        file: "tst_qmltyperegistrar.h"
+        name: "QPersistentModelIndexValueType"
+        accessSemantics: "value"
+        Property { name: "row"; type: "int"; read: "row"; index: 0; isReadonly: true; isFinal: true }
+    })"));
 }
 
 QTEST_MAIN(tst_qmltyperegistrar)
