@@ -18,6 +18,8 @@
 #include <QtQml/qqmlpropertyvaluesource.h>
 #include <QtQml/qqmlscriptstring.h>
 #include <QtQml/qqmlproperty.h>
+
+#include <private/qqmlcomponentattached_p.h>
 #include <private/qqmlcustomparser_p.h>
 
 QVariant myCustomVariantTypeConverter(const QString &data);
@@ -1547,6 +1549,7 @@ class BareSingleton : public QObject
     Q_OBJECT
     QML_SINGLETON
     QML_ELEMENT
+    QML_ADDED_IN_VERSION(1, 0)
 
 public:
     BareSingleton(QObject *parent = nullptr) : QObject(parent)
@@ -1560,6 +1563,7 @@ class UncreatableSingleton : public QObject
     Q_OBJECT
     QML_SINGLETON
     QML_ELEMENT
+    QML_ADDED_IN_VERSION(1, 0)
 
 public:
     static UncreatableSingleton *instance();
@@ -2405,6 +2409,20 @@ private:
     int m_length = 19;
 };
 
+struct ValueTypeWithString
+{
+    Q_GADGET
+    QML_VALUE_TYPE(withString)
+    QML_CONSTRUCTIBLE_VALUE
+
+public:
+    Q_INVOKABLE ValueTypeWithString(const QString &v = QString()) : m_string(v) {}
+    QString toString() const { return m_string; }
+
+private:
+    QString m_string;
+};
+
 class GetterObject : public QObject {
     Q_OBJECT
     QML_ELEMENT
@@ -2420,5 +2438,310 @@ public:
     Q_INVOKABLE quint64 getQTrue() const { return 1; }
 };
 
+namespace TypedEnums {
+Q_NAMESPACE
+QML_ELEMENT
+
+enum E8S : qint8 {
+    E8SA = std::numeric_limits<qint8>::min(),
+    E8SB = -5,
+    E8SC = -1,
+    E8SD = 0,
+    E8SE = 1,
+    E8SF = 5,
+    E8SG = std::numeric_limits<qint8>::max(),
+};
+Q_ENUM_NS(E8S);
+
+enum E8U : quint8 {
+    E8UA = 0,
+    E8UB = 1,
+    E8UC = 5,
+    E8UD = 1 << 7,
+    E8UE = std::numeric_limits<quint8>::max(),
+};
+Q_ENUM_NS(E8U);
+
+enum E16S : qint16 {
+    E16SA = std::numeric_limits<qint16>::min(),
+    E16SB = -5,
+    E16SC = -1,
+    E16SD = 0,
+    E16SE = 1,
+    E16SF = 5,
+    E16SG = std::numeric_limits<qint16>::max(),
+};
+Q_ENUM_NS(E16S);
+
+enum E16U : quint16 {
+    E16UA = 0,
+    E16UB = 1,
+    E16UC = 5,
+    E16UD = 1 << 15,
+    E16UE = std::numeric_limits<quint16>::max(),
+};
+Q_ENUM_NS(E16U);
+
+enum E32S : qint32 {
+    E32SA = std::numeric_limits<qint32>::min(),
+    E32SB = -5,
+    E32SC = -1,
+    E32SD = 0,
+    E32SE = 1,
+    E32SF = 5,
+    E32SG = std::numeric_limits<qint32>::max(),
+};
+Q_ENUM_NS(E32S);
+
+enum E32U : quint32 {
+    E32UA = 0,
+    E32UB = 1,
+    E32UC = 5,
+    E32UD = 1u << 31,
+    E32UE = std::numeric_limits<quint32>::max(),
+};
+Q_ENUM_NS(E32U);
+
+enum E64S : qint64 {
+    E64SA = std::numeric_limits<qint64>::min(),
+    E64SB = -5,
+    E64SC = -1,
+    E64SD = 0,
+    E64SE = 1,
+    E64SF = 5,
+    E64SG = std::numeric_limits<qint64>::max(),
+};
+Q_ENUM_NS(E64S);
+
+enum E64U : quint64 {
+    E64UA = 0,
+    E64UB = 1,
+    E64UC = 5,
+    E64UD = 1ull << 63,
+    E64UE = std::numeric_limits<quint64>::max(),
+};
+Q_ENUM_NS(E64U);
+}
+
+class GadgetWithEnums
+{
+    Q_GADGET
+    QML_VALUE_TYPE(gadgetWithEnums)
+    Q_PROPERTY(TypedEnums::E8S  e8s  MEMBER m_e8s);
+    Q_PROPERTY(TypedEnums::E8U  e8u  MEMBER m_e8u);
+    Q_PROPERTY(TypedEnums::E16S e16s MEMBER m_e16s);
+    Q_PROPERTY(TypedEnums::E16U e16u MEMBER m_e16u);
+    Q_PROPERTY(TypedEnums::E32S e32s MEMBER m_e32s);
+    Q_PROPERTY(TypedEnums::E32U e32u MEMBER m_e32u);
+    Q_PROPERTY(TypedEnums::E64S e64s MEMBER m_e64s);
+    Q_PROPERTY(TypedEnums::E64U e64u MEMBER m_e64u);
+public:
+    TypedEnums::E8S  m_e8s  = {};
+    TypedEnums::E8U  m_e8u  = {};
+    TypedEnums::E16S m_e16s = {};
+    TypedEnums::E16U m_e16u = {};
+    TypedEnums::E32S m_e32s = {};
+    TypedEnums::E32U m_e32u = {};
+    TypedEnums::E64S m_e64s = {};
+    TypedEnums::E64U m_e64u = {};
+private:
+    friend bool operator==(const GadgetWithEnums &a, const GadgetWithEnums &b)
+    {
+        return a.m_e8s == b.m_e8s && a.m_e8u == b.m_e8u && a.m_e16s == b.m_e16s
+                && a.m_e16u == b.m_e16u && a.m_e32s == b.m_e32s && a.m_e32u == b.m_e32u
+                && a.m_e64s == b.m_e64s && a.m_e64u == b.m_e64u;
+    }
+    friend bool operator!=(const GadgetWithEnums &a, const GadgetWithEnums &b)
+    {
+        return !(a == b);
+    }
+};
+
+class ObjectWithEnums : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(TypedEnums::E8S  e8s  MEMBER m_e8s  NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E8U  e8u  MEMBER m_e8u  NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E16S e16s MEMBER m_e16s NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E16U e16u MEMBER m_e16u NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E32S e32s MEMBER m_e32s NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E32U e32u MEMBER m_e32u NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E64S e64s MEMBER m_e64s NOTIFY changed);
+    Q_PROPERTY(TypedEnums::E64U e64u MEMBER m_e64u NOTIFY changed);
+    Q_PROPERTY(GadgetWithEnums g MEMBER m_g NOTIFY changed);
+public:
+    ObjectWithEnums(QObject *parent = nullptr) : QObject(parent) {}
+    TypedEnums::E8S  m_e8s  = {};
+    TypedEnums::E8U  m_e8u  = {};
+    TypedEnums::E16S m_e16s = {};
+    TypedEnums::E16U m_e16u = {};
+    TypedEnums::E32S m_e32s = {};
+    TypedEnums::E32U m_e32u = {};
+    TypedEnums::E64S m_e64s = {};
+    TypedEnums::E64U m_e64u = {};
+    GadgetWithEnums m_g;
+Q_SIGNALS:
+    void changed();
+};
+
+struct UnregisteredValueBaseType
+{
+    int foo = 12;
+};
+
+struct UnregisteredValueDerivedType: public UnregisteredValueBaseType
+{
+    int bar = 13;
+};
+
+struct GadgetedValueBaseType
+{
+    Q_GADGET
+    int foo = 12;
+};
+
+struct GadgetedValueDerivedType: public GadgetedValueBaseType
+{
+    Q_GADGET
+    int bar = 13;
+};
+
+class UnregisteredValueTypeHandler: public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+public:
+    int consumed = 0;
+    int gadgeted = 0;
+
+public slots:
+    UnregisteredValueBaseType produce() { return UnregisteredValueBaseType(); }
+    UnregisteredValueDerivedType produceDerived() { return UnregisteredValueDerivedType(); }
+    void consume(UnregisteredValueBaseType) { ++consumed; }
+
+    GadgetedValueDerivedType produceGadgeted() { return GadgetedValueDerivedType(); }
+    void consume(GadgetedValueBaseType) { ++gadgeted; }
+};
+
+class Greeter : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    Greeter(QObject *parent = nullptr) : QObject(parent) {}
+
+    Q_INVOKABLE void greet()
+    {
+        qDebug().noquote() << objectName() << "says hello";
+    }
+
+    Q_INVOKABLE void sum(int a, int b)
+    {
+        qDebug().noquote() << objectName() << QString("says %1 + %2 = %3").arg(a).arg(b).arg(a + b);
+    }
+};
+
+class Attachment : public QObject {
+    Q_OBJECT
+public:
+    Attachment(QObject *parent = nullptr) : QObject(parent) {}
+};
+
+class AttachedInCtor : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    QML_ATTACHED(Attachment)
+
+public:
+    AttachedInCtor(QObject *parent = nullptr)
+        : QObject(parent)
+    {
+        attached = qmlAttachedPropertiesObject<AttachedInCtor>(this, true);
+    }
+
+    static Attachment *qmlAttachedProperties(QObject *object) {
+        return new Attachment(object);
+    }
+
+    QObject *attached = nullptr;
+};
+
+class BirthdayParty : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(QQmlListProperty<QObject> guests READ guests)
+    Q_CLASSINFO("DefaultProperty", "guests")
+    QML_ELEMENT
+
+public:
+    using QObject::QObject;
+    QQmlListProperty<QObject> guests() { return {this, &m_guests}; }
+    qsizetype guestCount() const { return m_guests.count(); }
+    QObject *guest(qsizetype i) const { return m_guests.at(i); }
+
+private:
+    QList<QObject *> m_guests;
+};
+
+class ByteArrayReceiver : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    QList<QByteArray> byteArrays;
+
+    Q_INVOKABLE void byteArrayTest(const QByteArray &ba)
+    {
+        byteArrays.push_back(ba);
+    }
+};
+
+class CounterAttachedBaseType: public QObject
+{
+    Q_OBJECT
+    QML_ANONYMOUS
+    Q_PROPERTY (int value READ value NOTIFY valueChanged)
+
+public:
+    CounterAttachedBaseType(QObject *parent = nullptr) : QObject(parent) {}
+
+    int value() { return m_value; }
+    Q_SIGNAL void valueChanged();
+
+protected:
+    int m_value = 98;
+};
+
+
+class CounterAttachedType: public CounterAttachedBaseType
+{
+    Q_OBJECT
+    QML_ANONYMOUS
+
+public:
+    CounterAttachedType(QObject *parent = nullptr) : CounterAttachedBaseType(parent) {}
+
+    Q_INVOKABLE void increase() {
+        ++m_value;
+        Q_EMIT valueChanged();
+    }
+};
+
+class Counter : public QObject
+{
+    Q_OBJECT
+    QML_ATTACHED(CounterAttachedBaseType)
+    QML_ELEMENT
+
+public:
+    static CounterAttachedBaseType *qmlAttachedProperties(QObject *o)
+    {
+        return new CounterAttachedType(o);
+    }
+};
 
 #endif // TESTTYPES_H
